// Code written by daxnitro.  Do what you want with it but give me some credit if you use it in whole or in part.

package net.minecraft.src;

import net.minecraft.client.Minecraft;

import java.nio.*;

import org.lwjgl.opengl.*;
import org.lwjgl.BufferUtils;

import java.io.BufferedReader;
import java.io.InputStreamReader;

public class Shaders
{

	private Shaders() {
		mc = null;
		initShaders();
		initLightSources();
	}
	
	private void initShaders() {
		baseProgram = setupProgram("/shaders/base.vsh", "#define _ENABLE_GL_TEXTURE_2D\n", "/shaders/base.fsh", "#define _ENABLE_GL_TEXTURE_2D\n");
		baseProgramNoT2D = setupProgram("/shaders/base.vsh", "", "/shaders/base.fsh", "");
		baseProgramBM = setupProgram("/shaders/base.vsh", "#define _ENABLE_GL_TEXTURE_2D\n#define _ENABLE_BUMP_MAPPING\n", "/shaders/base.fsh", "#define _ENABLE_GL_TEXTURE_2D\n#define _ENABLE_BUMP_MAPPING\n");

		finalProgram = setupProgram("/shaders/final.vsh", "", "/shaders/final.fsh", "");
	}
	
	private int setupProgram(String vShaderPath, String vPrefix, String fShaderPath, String fPrefix) {
		int program = ARBShaderObjects.glCreateProgramObjectARB();
		
		int vShader = 0;
		int fShader = 0;
		
		if (program != 0) {
			vShader = createVertShader(vShaderPath, vPrefix);
			fShader = createFragShader(fShaderPath, fPrefix);
		}

		if (vShader != 0 || fShader != 0) {
			if (vShader != 0) {
				ARBShaderObjects.glAttachObjectARB(program, vShader);
			}
			if (fShader != 0) {
				ARBShaderObjects.glAttachObjectARB(program, fShader);
			}
			ARBShaderObjects.glLinkProgramARB(program);
			ARBShaderObjects.glValidateProgramARB(program);
			printLogInfo(program);
		} else {
			program = 0;
		}
		
		return program;
	}
	
	public void useProgram(int program) {
		ARBShaderObjects.glUseProgramObjectARB(program);
		activeProgram = program;
		if (program != 0) {
			int sampler1U = ARBShaderObjects.glGetUniformLocationARB(program, "sampler1");
			ARBShaderObjects.glUniform1iARB(sampler1U, 1);
			int sampler2U = ARBShaderObjects.glGetUniformLocationARB(program, "sampler2");
			ARBShaderObjects.glUniform1iARB(sampler2U, 2);
			int fogMode = GL11.glGetInteger(GL11.GL_FOG_MODE);
			int fogModeU = ARBShaderObjects.glGetUniformLocationARB(program, "fogMode");
			ARBShaderObjects.glUniform1iARB(fogModeU, fogMode);
			int renderTypeU = ARBShaderObjects.glGetUniformLocationARB(program, "renderType");
			ARBShaderObjects.glUniform1iARB(renderTypeU, renderType);
			int sunPositionU = ARBShaderObjects.glGetUniformLocationARB(program, "sunPosition");
			ARBShaderObjects.glUniform3fARB(sunPositionU, sunPosition[0], sunPosition[1], sunPosition[2]);
			int moonPositionU = ARBShaderObjects.glGetUniformLocationARB(program, "moonPosition");
			ARBShaderObjects.glUniform3fARB(moonPositionU, moonPosition[0], moonPosition[1], moonPosition[2]);
			if (mc != null) {
				int itemId;
				ItemStack stack = mc.thePlayer.inventory.getCurrentItem();
				if (stack != null && (itemId = stack.itemID) < lightSources.length && lightSources[itemId] != null) {
					int itemIdU = ARBShaderObjects.glGetUniformLocationARB(program, "heldLight.itemId");
					ARBShaderObjects.glUniform1iARB(itemIdU, itemId);
					int magnitudeU = ARBShaderObjects.glGetUniformLocationARB(program, "heldLight.magnitude");
					ARBShaderObjects.glUniform1fARB(magnitudeU, lightSources[itemId].magnitude);
					int specularU = ARBShaderObjects.glGetUniformLocationARB(program, "heldLight.specular");
					ARBShaderObjects.glUniform4fARB(specularU, lightSources[itemId].specular[0], lightSources[itemId].specular[1], lightSources[itemId].specular[2], lightSources[itemId].specular[3]);
				} else {
					int itemIdU = ARBShaderObjects.glGetUniformLocationARB(program, "heldLight.itemId");
					ARBShaderObjects.glUniform1iARB(itemIdU, -1);
					int magnitudeU = ARBShaderObjects.glGetUniformLocationARB(program, "heldLight.magnitude");
					ARBShaderObjects.glUniform1fARB(magnitudeU, 0.0F);
				}
				int worldTimeU = ARBShaderObjects.glGetUniformLocationARB(program, "worldTime");
				ARBShaderObjects.glUniform1iARB(worldTimeU, (int)(mc.theWorld.worldTime % 24000L));
				int aspectRatioU = ARBShaderObjects.glGetUniformLocationARB(program, "aspectRatio");
				ARBShaderObjects.glUniform1fARB(aspectRatioU, (float)mc.displayWidth / (float)mc.displayHeight);
				int displayWidthU = ARBShaderObjects.glGetUniformLocationARB(program, "displayWidth");
				ARBShaderObjects.glUniform1fARB(displayWidthU, (float)mc.displayWidth);
				int displayHeightU = ARBShaderObjects.glGetUniformLocationARB(program, "displayHeight");
				ARBShaderObjects.glUniform1fARB(displayHeightU, (float)mc.displayHeight);
				int nearU = ARBShaderObjects.glGetUniformLocationARB(program, "near");
				ARBShaderObjects.glUniform1fARB(nearU, 0.05F);
				int farU = ARBShaderObjects.glGetUniformLocationARB(program, "far");
				ARBShaderObjects.glUniform1fARB(farU, 256 >> mc.gameSettings.renderDistance);
			}
		}
	}
	
	private int createVertShader(String filename, String prefixCode) {
		int vertShader = ARBShaderObjects.glCreateShaderObjectARB(ARBVertexShader.GL_VERTEX_SHADER_ARB);
		if (vertShader == 0) {
			return 0;
		}
		String vertexCode = prefixCode;
		String line;
		try {
			BufferedReader reader = new BufferedReader(new InputStreamReader((Shaders.class).getResourceAsStream(filename)));
			while ((line=reader.readLine()) != null) {
				vertexCode += line + "\n";
			}
		} catch(Exception e) {
			return 0;
		}
		ARBShaderObjects.glShaderSourceARB(vertShader, vertexCode);
		ARBShaderObjects.glCompileShaderARB(vertShader);
		printLogInfo(vertShader);
		return vertShader;
	}
	 
	private int createFragShader(String filename, String prefixCode) {
		int fragShader = ARBShaderObjects.glCreateShaderObjectARB(ARBFragmentShader.GL_FRAGMENT_SHADER_ARB);
		if (fragShader == 0) {
			return 0;
		}
		String fragCode = prefixCode;
		String line;
		try {
			BufferedReader reader = new BufferedReader(new InputStreamReader((Shaders.class).getResourceAsStream(filename)));
			while ((line=reader.readLine()) != null) {
				fragCode += line + "\n";
			}
		} catch (Exception e) {
			return 0;
		}
		ARBShaderObjects.glShaderSourceARB(fragShader, fragCode);
		ARBShaderObjects.glCompileShaderARB(fragShader);
		printLogInfo(fragShader);
		return fragShader;
	}

	private static boolean printLogInfo(int obj) {
		IntBuffer iVal = BufferUtils.createIntBuffer(1);
		ARBShaderObjects.glGetObjectParameterARB(obj, ARBShaderObjects.GL_OBJECT_INFO_LOG_LENGTH_ARB, iVal);
		 
		int length = iVal.get();
		if (length > 1) {
			ByteBuffer infoLog = BufferUtils.createByteBuffer(length);
			iVal.flip();
			ARBShaderObjects.glGetInfoLogARB(obj, iVal, infoLog);
			byte[] infoBytes = new byte[length];
			infoLog.get(infoBytes);
			String out = new String(infoBytes);
			System.out.println("Info log:\n" + out);
			return false;
		}
		return true;
	}
	
	private int createTexture(int width, int height, boolean depth) throws java.lang.OutOfMemoryError {
		int textureId = GL11.glGenTextures();
		
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, textureId);
				
		if (depth) {
		   	ByteBuffer buffer = ByteBuffer.allocateDirect(width * height * 4 * 4);
			GL11.glTexImage2D(GL11.GL_TEXTURE_2D, 0, GL11.GL_DEPTH_COMPONENT, width, height, 0, GL11.GL_DEPTH_COMPONENT, GL11.GL_FLOAT, buffer);
		} else {
		   	ByteBuffer buffer = ByteBuffer.allocateDirect(width * height * 4);
			GL11.glTexImage2D(GL11.GL_TEXTURE_2D, 0, GL11.GL_RGBA, width, height, 0, GL11.GL_RGBA, GL11.GL_UNSIGNED_BYTE, buffer);
		}
		
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_LINEAR);
		GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_LINEAR);
		
		return textureId;
	}
	
	private void deleteTexture(int textureId) {
		GL11.glDeleteTextures(textureId);
	}

	public int baseTextureId() {
		if (baseTextureId == 0 || baseTextureWidth != mc.displayWidth || baseTextureHeight != mc.displayHeight) {
			try {
				int textureId = createTexture(mc.displayWidth, mc.displayHeight, false);
				baseTextureId = textureId;
				baseTextureWidth = mc.displayWidth;
				baseTextureHeight = mc.displayHeight;
			} catch (java.lang.OutOfMemoryError e) {
			}
		}
		return baseTextureId;
	}

	public int depthTextureId(Minecraft minecraft) {
		mc = minecraft;
		if (depthTextureId == 0 || depthTextureWidth != mc.displayWidth || depthTextureHeight != mc.displayHeight) {
			try {
				int textureId = createTexture(mc.displayWidth, mc.displayHeight, true);
				depthTextureId = textureId;
				depthTextureWidth = mc.displayWidth;
				depthTextureHeight = mc.displayHeight;
			} catch (java.lang.OutOfMemoryError e) {
			}
		}
		return depthTextureId;
	}

	public int depthTexture2Id(Minecraft minecraft) {
		mc = minecraft;
		if (depthTexture2Id == 0 || depthTexture2Width != mc.displayWidth || depthTexture2Height != mc.displayHeight) {
			try {
			int textureId = createTexture(mc.displayWidth, mc.displayHeight, true);
				depthTexture2Id = textureId;
				depthTexture2Width = mc.displayWidth;
				depthTexture2Height = mc.displayHeight;
			} catch (java.lang.OutOfMemoryError e) {
			}
		}
		return depthTexture2Id;
	}
	
	public void processScene(float red, float green, float blue) {
		if (!mc.gameSettings.anaglyph && finalProgram != 0) {
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, baseTextureId());
	  		GL11.glCopyTexImage2D(GL11.GL_TEXTURE_2D, 0, GL11.GL_RGBA, 0, 0, baseTextureWidth, baseTextureHeight, 0);
	  		GL13.glActiveTexture(GL13.GL_TEXTURE1);
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, depthTextureId(mc));
	  		GL13.glActiveTexture(GL13.GL_TEXTURE2);
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, depthTexture2Id(mc));
	  		GL13.glActiveTexture(GL13.GL_TEXTURE0);
			useProgram(finalProgram);
			GL11.glClearColor(red, green, blue, 0.0F);
			GL11.glClear(GL11.GL_COLOR_BUFFER_BIT);
			GL11.glDisable(GL11.GL_DEPTH_TEST);
			GL11.glDisable(GL11.GL_BLEND);
			GL11.glMatrixMode(5889 /*GL_PROJECTION*/);
			GL11.glLoadIdentity();
			GL11.glOrtho(0, mc.displayWidth, mc.displayHeight, 0, -1, 1);
			GL11.glMatrixMode(5888 /*GL_MODELVIEW0_ARB*/);
			GL11.glLoadIdentity();
			GL11.glBegin(GL11.GL_QUADS);
			GL11.glTexCoord2f(0, 1);
			GL11.glVertex3f(0, 0, 0);
			GL11.glTexCoord2f(0, 0);
			GL11.glVertex3f(0, mc.displayHeight, 0);
			GL11.glTexCoord2f(1, 0);
			GL11.glVertex3f(mc.displayWidth, mc.displayHeight, 0);
			GL11.glTexCoord2f(1, 1);
			GL11.glVertex3f(mc.displayWidth, 0, 0);
			GL11.glEnd();
			GL11.glEnable(GL11.GL_DEPTH_TEST);
			useProgram(0);
		}
	}
	
	public static void copyDepthTexture(int texture) {
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, texture);
		if (texture == depthTextureId) {
			GL11.glCopyTexImage2D(GL11.GL_TEXTURE_2D, 0, GL11.GL_DEPTH_COMPONENT, 0, 0, depthTextureWidth, depthTextureHeight, 0);
		} else if (texture == depthTexture2Id) {
			GL11.glCopyTexImage2D(GL11.GL_TEXTURE_2D, 0, GL11.GL_DEPTH_COMPONENT, 0, 0, depthTexture2Width, depthTexture2Height, 0);
		} else {
			GL11.glCopyTexImage2D(GL11.GL_TEXTURE_2D, 0, GL11.GL_DEPTH_COMPONENT, 0, 0, mc.displayWidth, mc.displayHeight, 0);
		}
	}
	
	public static void bindTexture(int activeTexture, int texture) {
		GL13.glActiveTexture(activeTexture);
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, texture);
		GL13.glActiveTexture(GL13.GL_TEXTURE0);
	}
	
	public void setRenderType(int type) {
		renderType = type;
		int renderTypeU = ARBShaderObjects.glGetUniformLocationARB(activeProgram, "renderType");
		ARBShaderObjects.glUniform1iARB(renderTypeU, renderType);
	}	
	
	public void setCelestialPosition() {
		// This is called when the current matrix is the modelview matrix based on the celestial angle.
		// The sun is at (0, 100, 0), and the moon is at (0, -100, 0).
		FloatBuffer modelView = ByteBuffer.allocateDirect(64).order(ByteOrder.nativeOrder()).asFloatBuffer();
		GL11.glGetFloat(GL11.GL_MODELVIEW_MATRIX, modelView);
		float[] mv = new float[16];
		modelView.get(mv, 0, 16);
		float[] sunPos = multiplyMat4xVec4(mv, new float[]{0.0F, 100.0F, 0.0F, 0.0F});
		sunPosition = sunPos;
		float[] moonPos = multiplyMat4xVec4(mv, new float[]{0.0F, -100.0F, 0.0F, 0.0F});
		moonPosition = moonPos;
	}
	
	private static float[] multiplyMat4xVec4(float[] ta, float[] tb) {
		float[] mout = new float[4];
		mout[0] = ta[0] * tb[0] + ta[4] * tb[1] +  ta[8] * tb[2] + ta[12] * tb[3];
		mout[1] = ta[1] * tb[0] + ta[5] * tb[1] +  ta[9] * tb[2] + ta[13] * tb[3];
		mout[2] = ta[2] * tb[0] + ta[6] * tb[1] + ta[10] * tb[2] + ta[14] * tb[3];
		mout[3] = ta[3] * tb[0] + ta[7] * tb[1] + ta[11] * tb[2] + ta[15] * tb[3];
		return mout;
	}
	
	private void initLightSources() {
		lightSources = new LightSource[Item.itemsList.length];
		for (int i = 0; i < Item.itemsList.length; ++i) {
			if (Item.itemsList[i] == null) {
				continue;
			}
			if (i < Block.blocksList.length && Block.blocksList[i] != null) {
				lightSources[i] = new LightSource(Block.lightValue[i]);
				if (Block.blocksList[i] == Block.torchWood) {
					lightSources[i].setSpecular(1.0F, 0.9F, 0.5F, 1.0F);
				} else if (Block.blocksList[i] == Block.torchRedstoneActive) {
					lightSources[i].setSpecular(1.0F, 0.0F, 0.0F, 1.0F);
				} else if (Block.blocksList[i] == Block.lightStone) {
					lightSources[i].setSpecular(1.0F, 1.0F, 0.8F, 1.0F);
				}
			} else if (Item.itemsList[i] == Item.bucketLava) {
				lightSources[i] = new LightSource(Block.lightValue[Block.lavaStill.blockID]);
				lightSources[i].setSpecular(1.0F, 0.5F, 0.0F, 1.0F);
			} else if (Item.itemsList[i] == Item.bucketWater) {
				lightSources[i] = new LightSource(2.0F);
				lightSources[i].setSpecular(0.0F, 0.0F, 0.3F, 1.0F);
			}
		}
	}
	
	private class LightSource {
		public LightSource(float magnitude) {
			this.magnitude = magnitude;
		}

		public void setSpecular(float r, float g, float b, float a) {
			specular[0] = r;
			specular[1] = g;
			specular[2] = b;
			specular[3] = a;
		}
		
		public float magnitude = 0.0F;
		public float[] specular = {1.0F, 1.0F, 1.0F, 1.0F};
	}
		
	public static final Shaders instance = new Shaders();
	public int activeProgram = 0;

	public int baseProgram = 0;
	public int baseProgramNoT2D = 0;
	public int baseProgramBM = 0;

	public int finalProgram = 0;
	
	private int renderType = 0; // RENDER_TYPE_UNKNOWN

	private float[] sunPosition = new float[3];
	private float[] moonPosition = new float[3];
	
	private LightSource[] lightSources;
	
	private static int baseTextureId = 0;
	private static int baseTextureWidth = 0;
	private static int baseTextureHeight = 0;

	private static int depthTextureId = 0;
	private static int depthTextureWidth = 0;
	private static int depthTextureHeight = 0;

	private static int depthTexture2Id = 0;
	private static int depthTexture2Width = 0;
	private static int depthTexture2Height = 0;
	
	public static Minecraft mc = null;

	public static final int RENDER_TYPE_UNKNOWN = 0;
	public static final int RENDER_TYPE_TERRAIN = 1;
}
